import CoreMotion
import SQLite3

class ActivityManager {
    let motionActivityManager = CMMotionActivityManager()
    // Create an instance of UserDefaultsManager
      let userDefaultsManager = UserDefaultsManager.shared

      let bikeCO2 = 255.0
      let runningCO2 = 265.0
      let walkCO2 = 271.0

      let perMinBikeDistance = 0.246
      let perMinBicycleDistance = 0.150
      let perMinRunningDistance = 0.140
      let perMinWalkDistance = 0.07435

    func queryActivityStarting(
        from start: Date,
        to end: Date,
        to queue: OperationQueue,
        withHandler handler: @escaping CMMotionActivityQueryHandler
    ) {
        motionActivityManager.queryActivityStarting(from: start, to: end, to: queue, withHandler: handler)
    }


    func queryActivityData(from startDate: Date, to endDate: Date = Date()) {
            let operationQueue = OperationQueue()
            operationQueue.maxConcurrentOperationCount = 1 // Ensure only one operation is executed at a time

            queryActivityStarting(from: startDate, to: endDate, to: operationQueue) { [self] activities, error in
                if let error = error {
                    print("ActivityManager Error querying activity data: \(error.localizedDescription)")
                    return
                }

                guard let activities = activities else {
                    print("ActivityManager No activity data available")
                    return
                }

                for activity in activities {
                    let actData = ActivityData(from: activity)
                    print("ActivityManager raw activity data: \(activity)")
                    print("ActivityManager activity type: \(actData.type), confidence: \(actData.confidence), startDate: \(actData.startDate)")

                    // Retrieve the boolean value from UserDefaults
                    let autoTrackingStatus = self.userDefaultsManager.autoTrackingOnOff
                    print("ActivityManager UserDefaults autoTrackingStatus:", autoTrackingStatus)

                    // add point calculation logic here
                    if(autoTrackingStatus && actData.type != "UNKNOWN"){
                          print("ActivityManager \(actData.type) activity tracked ...")
                          //todo write point calculation logic here
                                let recordedActivity = self.userDefaultsManager.recordedActivity
                                let recordedActivityTime = self.userDefaultsManager.recordedActivityTime

                                 print("ActivityManager activity type: \(actData.type), confidence: \(actData.confidence), startDate: \(actData.startDate)")
                                 print("ActivityManager userDefaultsManager recordedActivity =====> \(recordedActivity)")
                                 print("ActivityManager userDefaultsManager recordedActivityTime =====> \(recordedActivityTime)")

                                 if(recordedActivity == ""){
                                  print("ActivityManager =====> recordedActivity == null")
                                  self.userDefaultsManager.recordedActivity = actData.type
                                  self.userDefaultsManager.recordedActivityTime = actData.startDate

                                 }else{
                                 print("ActivityManager =====> recordedActivity != null")
                                  if (actData.type != recordedActivity) {
                                     print("ActivityManager =====> activityData != recordedActivity")

                                     let stillActivityTime = self.userDefaultsManager.stillActivityTime
                                     let timeIntervalInSeconds = Date().timeIntervalSince(stillActivityTime)

                                     if(actData.type == "STILL" && timeIntervalInSeconds <= 60){
                                     self.userDefaultsManager.stillActivityTime = Date()
                                     }else{
                                      self.userDefaultsManager.recordedActivity = actData.type
                                      self.userDefaultsManager.recordedActivityTime = Date()
                                      let timeIntervalInSeconds = Date().timeIntervalSince(recordedActivityTime)
                                      let timeIntervalInMinutes = Int(timeIntervalInSeconds / 60)
                                      print("ActivityManager mini difference is \(timeIntervalInSeconds) seconds.")
                                      print("ActivityManager mini difference is \(timeIntervalInMinutes) minutes.")
                                      print("ActivityManager mini recordedActivity \(recordedActivity)")
                                         switch recordedActivity {
                                         case "IN_VEHICLE":
                                                let distance = Int(Double(timeIntervalInMinutes) * self.perMinBikeDistance)
                                                let co2Vehicle = Int(Double(distance) * self.bikeCO2 / 1000.0)
                                                let points = Int(1.2 * Double(distance) * Double(co2Vehicle))
                                                UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge]) { granted, error in
                                                    if granted {
                                                        print("Notification authorization granted")
                                                        self.sendNotification(recordedActivity:recordedActivity,points:points)
                                                    } else {
                                                        print("Notification authorization denied")
                                                    }
                                                }
                                                //let points = Int(Double(timeIntervalInMinutes) * 0.29)
                                                let records =  DatabaseManager.shared.getActivityPoints()
                                                let numberOfRecords = records.count
                                                print("ActivityManager IN_VEHICLE records retrieved: \(numberOfRecords)")
                                                // Check if there are any records returned.
                                                if let firstRecord = records.first {
                                                    let vehiclePointsOfFirstRecord = firstRecord.vehiclePoints
                                                    print("ActivityManager IN_VEHICLE points: \(vehiclePointsOfFirstRecord)")
                                                    // edit record logic
                                                    DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                                                 vehiclePoints: firstRecord.vehiclePoints + points,
                                                                                                 bicyclePoints: firstRecord.bicyclePoints,
                                                                                                 runningPoints: firstRecord.runningPoints,
                                                                                                 walkingPoints: firstRecord.walkingPoints,
                                                                                                 stillPoints: firstRecord.stillPoints)
                                                } else {
                                                    print("ActivityManager IN_VEHICLE No records retrieved.")
                                                    // add record logic
                                                    DatabaseManager.shared.insertActivityPoints(vehiclePoints:points,
                                                    bicyclePoints:0,
                                                    runningPoints:0,
                                                    walkingPoints:0,
                                                    stillPoints:0)
                                                }
                                         case "ON_BICYCLE":
                                             let distance = Int(Double(timeIntervalInMinutes) * self.perMinBikeDistance)
                                             let co2Vehicle = Int(Double(distance) * self.bikeCO2 / 1000.0)
                                             let points = Int(1.2 * Double(distance) * Double(co2Vehicle))
                                             UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge]) { granted, error in
                                                 if granted {
                                                     print("Notification authorization granted")
                                                     self.sendNotification(recordedActivity:recordedActivity,points:points)
                                                 } else {
                                                     print("Notification authorization denied")
                                                 }
                                             }
                                             //let points = Int(Double(timeIntervalInMinutes) * 0.50) // per minute points for bicycle
                                             let records =  DatabaseManager.shared.getActivityPoints()
                                             let numberOfRecords = records.count
                                             print("ActivityManager ON_BICYCLE records retrieved: \(numberOfRecords)")
                                             // Check if there are any records returned.
                                             if let firstRecord = records.first {
                                                 let vehiclePointsOfFirstRecord = firstRecord.bicyclePoints
                                                 print("ActivityManager ON_BICYCLE points: \(vehiclePointsOfFirstRecord)")
                                                 // edit record logic
                                                 DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                                             vehiclePoints: firstRecord.vehiclePoints,
                                                                                             bicyclePoints: firstRecord.bicyclePoints + points,
                                                                                             runningPoints: firstRecord.runningPoints,
                                                                                             walkingPoints: firstRecord.walkingPoints,
                                                                                             stillPoints: firstRecord.stillPoints)
                                             } else {
                                                 print("ActivityManager ON_BICYCLE No records retrieved.")
                                                 // add record logic
                                                 DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                           bicyclePoints:points,
                                                                           runningPoints:0,
                                                                           walkingPoints:0,
                                                                           stillPoints:0)
                                             }
                                         case "RUNNING":
                                             let distance = Int(Double(timeIntervalInMinutes) * self.perMinRunningDistance)
                                             let co2Running = Int(Double(distance) * self.runningCO2 / 1000.0)
                                             let points = Int(1.5 * Double(distance) * Double(co2Running))
                                             UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge]) { granted, error in
                                                 if granted {
                                                     print("Notification authorization granted")
                                                     self.sendNotification(recordedActivity:recordedActivity,points:points)
                                                 } else {
                                                     print("Notification authorization denied")
                                                 }
                                             }
                                             //let points = Int(Double(timeIntervalInMinutes) * 0.45) // per minute points for running
                                             let records =  DatabaseManager.shared.getActivityPoints()
                                             let numberOfRecords = records.count
                                             print("ActivityManager RUNNING records retrieved: \(numberOfRecords)")
                                             // Check if there are any records returned.
                                             if let firstRecord = records.first {
                                                 let vehiclePointsOfFirstRecord = firstRecord.runningPoints
                                                 print("ActivityManager RUNNING points: \(vehiclePointsOfFirstRecord)")
                                                 // edit record logic
                                                 DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                                            vehiclePoints: firstRecord.vehiclePoints,
                                                                                            bicyclePoints: firstRecord.bicyclePoints,
                                                                                            runningPoints: firstRecord.runningPoints + points,
                                                                                            walkingPoints: firstRecord.walkingPoints,
                                                                                            stillPoints: firstRecord.stillPoints)
                                             } else {
                                                 print("ActivityManager RUNNING No records retrieved.")
                                                 // add record logic
                                                 DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                                                  bicyclePoints:0,
                                                                                                  runningPoints:points,
                                                                                                  walkingPoints:0,
                                                                                                  stillPoints:0)
                                             }
                                         case "WALKING":
                                             let distance = Int(Double(timeIntervalInMinutes) * self.perMinRunningDistance)
                                             let co2Running = Int(Double(distance) * self.runningCO2 / 1000.0)
                                             let points = Int(1.5 * Double(distance) * Double(co2Running))
                                             UNUserNotificationCenter.current().requestAuthorization(options: [.alert, .sound, .badge]) { granted, error in
                                                 if granted {
                                                     print("Notification authorization granted")
                                                     self.sendNotification(recordedActivity:recordedActivity,points:points)
                                                 } else {
                                                     print("Notification authorization denied")
                                                 }
                                             }
                                             //let points = Int(Double(timeIntervalInMinutes) * 0.3) // per minute points for walking
                                             let records =  DatabaseManager.shared.getActivityPoints()
                                             let numberOfRecords = records.count
                                             print("ActivityManager WALKING records retrieved: \(numberOfRecords)")
                                             // Check if there are any records returned.
                                             if let firstRecord = records.first {
                                                 let vehiclePointsOfFirstRecord = firstRecord.walkingPoints
                                                 print("ActivityManager WALKING points: \(vehiclePointsOfFirstRecord)")
                                                 // edit record logic
                                                 DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                                            vehiclePoints: firstRecord.vehiclePoints,
                                                                                            bicyclePoints: firstRecord.bicyclePoints,
                                                                                            runningPoints: firstRecord.runningPoints,
                                                                                            walkingPoints: firstRecord.walkingPoints + points,
                                                                                            stillPoints: firstRecord.stillPoints)
                                             } else {
                                                 print("ActivityManager WALKING No records retrieved.")
                                                 // add record logic
                                                 DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                                            bicyclePoints:0,
                                                                                            runningPoints:0,
                                                                                            walkingPoints:points,
                                                                                            stillPoints:0)
                                             }
                                         case "STILL":
                                             let distance = Int(Double(timeIntervalInMinutes) * self.perMinRunningDistance)
                                             let co2Running = Int(Double(distance) * self.runningCO2 / 1000.0)
                                             let points = Int(1.5 * Double(distance) * Double(co2Running))
                                             let points = 0
                                             let records =  DatabaseManager.shared.getActivityPoints()
                                             let numberOfRecords = records.count
                                             print("ActivityManager STILL records retrieved: \(numberOfRecords)")
                                             if let firstRecord = records.first {
                                                 let vehiclePointsOfFirstRecord = firstRecord.stillPoints
                                                 print("ActivityManager STILL points: \(vehiclePointsOfFirstRecord)")
                                                 // edit record logic
                                                 DatabaseManager.shared.updateActivityPoints(id: firstRecord.id,
                                                                                            vehiclePoints: firstRecord.vehiclePoints,
                                                                                            bicyclePoints: firstRecord.bicyclePoints,
                                                                                            runningPoints: firstRecord.runningPoints,
                                                                                            walkingPoints: firstRecord.walkingPoints,
                                                                                            stillPoints: firstRecord.stillPoints)
                                             } else {
                                                 print("ActivityManager STILL No records retrieved.")
                                                 // add record logic
                                                 DatabaseManager.shared.insertActivityPoints(vehiclePoints:0,
                                                                                            bicyclePoints:0,
                                                                                            runningPoints:0,
                                                                                            walkingPoints:0,
                                                                                            stillPoints:0)
                                             }
                                         default:
                                             print("UNKNOWN activity")
                                         }
                                     }

                                  }
                                 }
                            DatabaseManager.shared.insertActivityData(type: actData.type, confidence: actData.confidence,startDate:actData.startDate)
                          }else{
                          print("ActivityManager UNKNOWN activity tracked ...")
                          }
                }

            }
    }


    func sendNotification(recordedActivity: String, points: Int) {
          // Create notification content
          let content = UNMutableNotificationContent()
          content.title = "Activity tracked"
          content.body = "\(recordedActivity) activity tracked with \(points) points"
          content.sound = UNNotificationSound.default

          // Create trigger for notification
          let trigger = UNTimeIntervalNotificationTrigger(timeInterval: 5, repeats: false)

          // Generate a unique identifier for the notification request
          let identifier = UUID().uuidString
          let currentTimeMillis = getCurrentTimeMillis()
          print("Current time in milliseconds: \(currentTimeMillis)")

          // Create notification request
          let request = UNNotificationRequest(identifier: currentTimeMillis, content: content, trigger: trigger)

          // Schedule notification
          UNUserNotificationCenter.current().add(request) { error in
              if let error = error {
                  print("Failed to Notification schedule : \(error)")
              } else {
                  print("Notification schedule successfully")
              }
          }
      }

      func getCurrentTimeMillis() -> String {
          let currentDate = Date()
          let milliseconds = Int64(currentDate.timeIntervalSince1970 * 1000)
          return "\(milliseconds)"
      }
}
