import Foundation
import SQLite3

class DatabaseManager {
    static let shared = DatabaseManager() // Singleton instance

    private var db: OpaquePointer? = nil

    private init() {
        openDatabase()
    }

    private func openDatabase() {
        let dbPath = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first! + "/myDatabase.db"

        if sqlite3_open(dbPath, &db) == SQLITE_OK {
            print("DatabaseManager Successfully opened connection to database.")
        } else {
            let errorMessage = String(cString: sqlite3_errmsg(db)!)
            print("DatabaseManager Unable to open database. Error: \(errorMessage)")
        }
    }

    func insertActivityData(type: String, confidence: String, startDate:Date) {
         print("DatabaseManager activity type: \(type), confidence: \(confidence)")
         let insertStatementString = "INSERT INTO activity (id, name, confidence, lastUpdatedTime) VALUES (?, ?, ?, ?);"
         var insertStatement: OpaquePointer? = nil

         // Create a DateFormatter instance
         let dateFormatter = DateFormatter()
         dateFormatter.dateFormat = "dd MMM yyyy hh:mm:ss a"
         let startDateString = dateFormatter.string(from: startDate)
         print("converted startDateString: \(startDateString)")

         if sqlite3_prepare_v2(db, insertStatementString, -1, &insertStatement, nil) == SQLITE_OK {
             let columnID = String(Date().timeIntervalSince1970 * 1000)
             let columnNAME = type
             let columnConfidence = confidence
             let columnLastUpTime = startDateString
             let SQLITE_TRANSIENT = unsafeBitCast(-1, to: sqlite3_destructor_type.self)
             sqlite3_bind_text(insertStatement, 2, columnNAME.cString(using: .utf8), -1, SQLITE_TRANSIENT)
             sqlite3_bind_text(insertStatement, 3, columnConfidence.cString(using: .utf8), -1, SQLITE_TRANSIENT)
             sqlite3_bind_text(insertStatement, 4, columnLastUpTime.cString(using: .utf8), -1, SQLITE_TRANSIENT)


             if sqlite3_step(insertStatement) == SQLITE_DONE {
                 print("DatabaseManager Successfully inserted row.")
             } else {
                 let errorMessage = String(cString: sqlite3_errmsg(db)!)
                 print("DatabaseManager Failed to insert row. Error: \(errorMessage)")
             }
         } else {
             let errorMessage = String(cString: sqlite3_errmsg(db)!)
             print("DatabaseManager Failed to prepare statement. Error: \(errorMessage)")
         }

         sqlite3_finalize(insertStatement)
    }

    func insertActivityPoints(vehiclePoints: Int, bicyclePoints: Int, runningPoints:Int, walkingPoints:Int, stillPoints:Int) {
             print("DatabaseManager activity points vehiclePoints: \(vehiclePoints), bicyclePoints: \(bicyclePoints), runningPoints: \(runningPoints), walkingPoints: \(walkingPoints), stillPoints: \(stillPoints)")
             let insertStatementString = "INSERT INTO tableActivityPoints (id, invehiclepoints, onbicyclepoints, runningpoints, walkingpoints, stillpoints) VALUES (?, ?, ?, ?, ?, ?);"
             var insertStatement: OpaquePointer? = nil

             if sqlite3_prepare_v2(db, insertStatementString, -1, &insertStatement, nil) == SQLITE_OK {
                 let columnID = String(Date().timeIntervalSince1970 * 1000)
                 let columnInVehiclePoints = vehiclePoints
                 let columnOnBicyclePoints = bicyclePoints
                 let columnRunningPoints = runningPoints
                 let columnWalkingPoints = walkingPoints
                 let columnStillPoints = stillPoints

                 let SQLITE_TRANSIENT = unsafeBitCast(-1, to: sqlite3_destructor_type.self)
                 sqlite3_bind_int(insertStatement, 2, Int32(columnInVehiclePoints))
                 sqlite3_bind_int(insertStatement, 3, Int32(columnOnBicyclePoints))
                 sqlite3_bind_int(insertStatement, 4, Int32(columnRunningPoints))
                 sqlite3_bind_int(insertStatement, 5, Int32(columnWalkingPoints))
                 sqlite3_bind_int(insertStatement, 6, Int32(columnStillPoints))


//                  sqlite3_bind_text(insertStatement, 2, columnInVehiclePoints.cInt(using: .utf8), -1, SQLITE_TRANSIENT)
//                  sqlite3_bind_text(insertStatement, 3, columnOnBicyclePoints.cInt(using: .utf8), -1, SQLITE_TRANSIENT)
//                  sqlite3_bind_text(insertStatement, 4, columnRunningPoints.cInt(using: .utf8), -1, SQLITE_TRANSIENT)
//                  sqlite3_bind_text(insertStatement, 5, columnWalkingPoints.cInt(using: .utf8), -1, SQLITE_TRANSIENT)
//                  sqlite3_bind_text(insertStatement, 6, columnStillPoints.cInt(using: .utf8), -1, SQLITE_TRANSIENT)


                 if sqlite3_step(insertStatement) == SQLITE_DONE {
                     print("DatabaseManager Successfully inserted row.")
                 } else {
                     let errorMessage = String(cString: sqlite3_errmsg(db)!)
                     print("DatabaseManager Failed to insert row. Error: \(errorMessage)")
                 }
             } else {
                 let errorMessage = String(cString: sqlite3_errmsg(db)!)
                 print("DatabaseManager Failed to prepare statement. Error: \(errorMessage)")
             }

             sqlite3_finalize(insertStatement)
        }

        func updateActivityPoints(id: Int, vehiclePoints: Int, bicyclePoints: Int, runningPoints: Int, walkingPoints: Int, stillPoints: Int) {
            print("DatabaseManager updating record with ID: \(id)")

            let updateStatementString = "UPDATE tableActivityPoints SET invehiclepoints = ?, onbicyclepoints = ?, runningpoints = ?, walkingpoints = ?, stillpoints = ? WHERE id = ?;"
            var updateStatement: OpaquePointer? = nil

            if sqlite3_prepare_v2(db, updateStatementString, -1, &updateStatement, nil) == SQLITE_OK {
                sqlite3_bind_int(updateStatement, 1, Int32(vehiclePoints))
                sqlite3_bind_int(updateStatement, 2, Int32(bicyclePoints))
                sqlite3_bind_int(updateStatement, 3, Int32(runningPoints))
                sqlite3_bind_int(updateStatement, 4, Int32(walkingPoints))
                sqlite3_bind_int(updateStatement, 5, Int32(stillPoints))
                sqlite3_bind_int(updateStatement, 6, Int32(id))

                if sqlite3_step(updateStatement) == SQLITE_DONE {
                    print("DatabaseManager Successfully updated row with ID: \(id)")
                } else {
                    let errorMessage = String(cString: sqlite3_errmsg(db)!)
                    print("DatabaseManager Failed to update row. Error: \(errorMessage)")
                }
            } else {
                let errorMessage = String(cString: sqlite3_errmsg(db)!)
                print("DatabaseManager Failed to prepare statement. Error: \(errorMessage)")
            }

            sqlite3_finalize(updateStatement)
        }

        func deleteActivityPoints(id: Int) {
            print("DatabaseManager deleting record with ID: \(id)")

            let deleteStatementString = "DELETE FROM tableActivityPoints WHERE id = ?;"
            var deleteStatement: OpaquePointer? = nil

            if sqlite3_prepare_v2(db, deleteStatementString, -1, &deleteStatement, nil) == SQLITE_OK {
                sqlite3_bind_int(deleteStatement, 1, Int32(id))

                if sqlite3_step(deleteStatement) == SQLITE_DONE {
                    print("DatabaseManager Successfully deleted row with ID: \(id)")
                } else {
                    let errorMessage = String(cString: sqlite3_errmsg(db)!)
                    print("DatabaseManager Failed to delete row. Error: \(errorMessage)")
                }
            } else {
                let errorMessage = String(cString: sqlite3_errmsg(db)!)
                print("DatabaseManager Failed to prepare statement. Error: \(errorMessage)")
            }

            sqlite3_finalize(deleteStatement)
        }

        func getActivityPoints() -> [(id: Int, vehiclePoints: Int, bicyclePoints: Int, runningPoints: Int, walkingPoints: Int, stillPoints: Int)] {
            var records: [(id: Int, vehiclePoints: Int, bicyclePoints: Int, runningPoints: Int, walkingPoints: Int, stillPoints: Int)] = []

            let queryStatementString = "SELECT id, invehiclepoints, onbicyclepoints, runningpoints, walkingpoints, stillpoints FROM tableActivityPoints;"
            var queryStatement: OpaquePointer? = nil

            if sqlite3_prepare_v2(db, queryStatementString, -1, &queryStatement, nil) == SQLITE_OK {
                while sqlite3_step(queryStatement) == SQLITE_ROW {
                    let id = Int(sqlite3_column_int(queryStatement, 0))
                    let vehiclePoints = Int(sqlite3_column_int(queryStatement, 1))
                    let bicyclePoints = Int(sqlite3_column_int(queryStatement, 2))
                    let runningPoints = Int(sqlite3_column_int(queryStatement, 3))
                    let walkingPoints = Int(sqlite3_column_int(queryStatement, 4))
                    let stillPoints = Int(sqlite3_column_int(queryStatement, 5))

                    records.append((id, vehiclePoints, bicyclePoints, runningPoints, walkingPoints, stillPoints))
                }
            } else {
                let errorMessage = String(cString: sqlite3_errmsg(db)!)
                print("DatabaseManager Failed to prepare statement. Error: \(errorMessage)")
            }

            sqlite3_finalize(queryStatement)

            return records
        }



}
